/*
 * SearchJTextField.java
 * Created on 2011/10/13
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

package jp.co.ntt.oss.heapstats.gui;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * extends {@link JTextField}<br>
 * implements {@link ListSelectionListener}<br>
 * <br>
 * This class performs an incremental regular expressions.
 */
public class SearchJTextField extends JTextField implements
                                                ListSelectionListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = 3403680650863473889L;

    /** Regular expression to narrow the class. */
    private DoSearch search;

    /** The DefaultListModel Class Path list. */
    private DefaultListModel searchListModel;

    /**
     * Creates a SearchJTextField for incremental research.
     *
     * @param col the number of columns to use to calculate the preferred width;
     *        if columns is set to zero, the preferred width will be whatever
     *        naturally results from the component implementation
     * @param classModel The DefaultListModel Class Path list
     */
    public SearchJTextField(final int col, final DefaultListModel classModel) {
        super(col);
        search = new DoSearch();
        searchListModel = classModel;
    }

    /**
     * Add the classpath to be searched.
     *
     * @param list Class path list
     */
    public final void addList(final Set<String> list) {
        search.addList(list);
    }

    /**
     * Clear the list of search criteria.
     */
    public final void clear() {
        setText("");
        search.clearCompletions();
    }

    @Override
    public final void valueChanged(final ListSelectionEvent e) {
    }

    /**
     * implements {@link DocumentListener}<br>
     * <br>
     * The class to retrieve the path to match the regular expression classes.
     */
    class DoSearch implements
                  DocumentListener {
        /** Search list of all classes. */
        private List<String> classListSearch;

        /**
         * Create a DoSearch to get the class path expression.
         */
        public DoSearch() {
            classListSearch = new ArrayList<String>();
            getDocument().addDocumentListener(this);
        }

        /**
         * Add a list to be searched.
         *
         * @param list A list of classpath
         */
        public void addList(final Set<String> list) {
            classListSearch.clear();
            classListSearch.addAll(list);
            incrementalSearch();
        }

        /**
         * Clear the information in this class.
         */
        public void clearCompletions() {
            classListSearch.clear();
            searchListModel.clear();
        }

        /**
         * Narrow down the string using regular expression entered.
         */
        private void incrementalSearch() {
            searchListModel.clear();
            searchListModel.setSize(0);
            if (getText().length() < 1) {
                return;
            }

            StringBuilder inputBuf = new StringBuilder();

            inputBuf.append(".*");
            for (int i = 0; i < getText().length(); i++) {
                String last = getText().substring(i, i + 1);
                if (last.matches("[0-9a-zA-Z]")) {
                    inputBuf.append(last);
                } else {
                    inputBuf.append("\\");
                    inputBuf.append(last);
                }
            }
            inputBuf.append(".*");

            Pattern pattern = Pattern.compile(inputBuf.toString(),
                    Pattern.CASE_INSENSITIVE);

            for (String completion : classListSearch) {
                // check if match
                Matcher matcher = pattern.matcher(completion);
                if (matcher.matches()) {
                    // add if match
                    searchListModel.addElement(completion);
                }
            }
        }

        @Override
        public void insertUpdate(final DocumentEvent e) {
            incrementalSearch();
        }

        @Override
        public void removeUpdate(final DocumentEvent e) {
            incrementalSearch();
        }

        @Override
        public void changedUpdate(final DocumentEvent e) {
        }
    }
}
